==> OpenCL-0.8.3.tgz: OpenCL-0.8.3/checks/bigint.cpp <==
#include <vector>
#include <string>
#include <fstream>
#include <iostream>
#include <cstdlib>

#include <opencl/bigint.h>
using namespace OpenCL;

std::vector<std::string> parse(const std::string&);
void strip(std::string&);

u32bit check_add(const std::vector<std::string>&);
u32bit check_sub(const std::vector<std::string>&);
u32bit check_mul(const std::vector<std::string>&);
u32bit check_div(const std::vector<std::string>&);
u32bit check_mod(const std::vector<std::string>&);
u32bit check_shr(const std::vector<std::string>&);
u32bit check_shl(const std::vector<std::string>&);

int do_bigint_tests(const std::string& filename)
   {
   std::ifstream test_data(filename.c_str());

   if(!test_data)
       {
       std::cout << "Couldn't open test file " << filename << std::endl;
       std::exit(1);
       }

   int errors = 0, alg_count = 0;
   std::string algorithm;

   while(!test_data.eof())
      {
      if(test_data.bad() || test_data.fail())
         {
         std::cout << "File I/O error." << std::endl;
         std::exit(1);
         }

      std::string line;
      std::getline(test_data, line);

      strip(line);
      if(line.size() == 0) continue;

      // Do line continuation
      while(line[line.size()-1] == '\\' && !test_data.eof())
         {
         line.replace(line.size()-1, 1, "");
         std::string nextline;
         std::getline(test_data, nextline);
         strip(nextline);
         if(nextline.size() == 0) continue;
         line += nextline;
         }

      if(line[0] == '[' && line[line.size() - 1] == ']')
         {
         algorithm = line.substr(1, line.size() - 2);
         alg_count = 0;
         continue;
         }

      std::vector<std::string> substr = parse(line);

      u32bit new_errors = 0;
      if(algorithm.find("Addition") != std::string::npos)
         new_errors = check_add(substr);

      if(algorithm.find("Subtraction") != std::string::npos)
         new_errors = check_sub(substr);

      if(algorithm.find("Multiplication") != std::string::npos)
         new_errors = check_mul(substr);

      if(algorithm.find("Division") != std::string::npos)
         new_errors = check_div(substr);

      if(algorithm.find("Modulo") != std::string::npos)
         new_errors = check_mod(substr);

      if(algorithm.find("LeftShift") != std::string::npos)
         new_errors = check_shl(substr);

      if(algorithm.find("RightShift") != std::string::npos)
         new_errors = check_shr(substr);

      alg_count++;
      errors += new_errors;

      if(new_errors)
         std::cout << "ERROR: BigInt " << algorithm << " failed test #"
                   << alg_count << std::endl;
      }

   return errors;
   }

u32bit check_add(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a + b;
   BigInt e = a;
   e += b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(d != e)
         std::cout << "ERROR: operator+/operator+= mismatch" << std::endl;
      return 1;
      }
   }

u32bit check_sub(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a - b;
   BigInt e = a;
   e -= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(d != e)
         std::cout << "ERROR: operator-/operator-= mismatch" << std::endl;
      return 1;
      }
   }

u32bit check_mul(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a * b;
   BigInt e = a;
   e *= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(d != e)
         std::cout << "ERROR: operator*/operator*= mismatch" << std::endl;
      return 1;
      }

   }

u32bit check_div(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a / b;
   BigInt e = a;
   e /= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator/ / operator/= mismatch" << std::endl;
      return 1;
      }

   }

u32bit check_mod(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a % b;
   BigInt e = a;
   e %= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator%/operator%= mismatch" << std::endl;
      return 1;
      }

   }

u32bit check_shl(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   u32bit b = std::atoi(args[1].c_str());
   BigInt c(args[2]);

   BigInt d = a << b;
   BigInt e = a;
   e <<= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator<</operator<<= mismatch" << std::endl;
      return 1;
      }

   }

u32bit check_shr(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   u32bit b = atoi(args[1].c_str());
   BigInt c(args[2]);

   BigInt d = a >> b;
   BigInt e = a;
   e >>= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator>>/operator>>= mismatch" << std::endl;
      std::cout << c << std::endl;
      std::cout << d << std::endl;
      std::cout << e << std::endl;
      return 1;
      }
   }

==> OpenCL-0.8.5.tgz: OpenCL-0.8.5/checks/bigint.cpp <==
#include <vector>
#include <string>
#include <fstream>
#include <iostream>
#include <cstdlib>

#include <opencl/bigint.h>
using namespace OpenCL;

#define DEBUG 0

std::vector<std::string> parse(const std::string&);
void strip(std::string&);

u32bit check_add(const std::vector<std::string>&);
u32bit check_sub(const std::vector<std::string>&);
u32bit check_mul(const std::vector<std::string>&);
u32bit check_div(const std::vector<std::string>&);
u32bit check_mod(const std::vector<std::string>&);
u32bit check_shr(const std::vector<std::string>&);
u32bit check_shl(const std::vector<std::string>&);

u32bit do_bigint_tests(const std::string& filename)
   {
   std::ifstream test_data(filename.c_str());

   if(!test_data)
       {
       std::cout << "Couldn't open test file " << filename << std::endl;
       std::exit(1);
       }

   u32bit errors = 0, alg_count = 0;
   std::string algorithm;

   while(!test_data.eof())
      {
      if(test_data.bad() || test_data.fail())
         {
         std::cout << "File I/O error." << std::endl;
         std::exit(1);
         }

      std::string line;
      std::getline(test_data, line);

      strip(line);
      if(line.size() == 0) continue;

      // Do line continuation
      while(line[line.size()-1] == '\\' && !test_data.eof())
         {
         line.replace(line.size()-1, 1, "");
         std::string nextline;
         std::getline(test_data, nextline);
         strip(nextline);
         if(nextline.size() == 0) continue;
         line += nextline;
         }

      if(line[0] == '[' && line[line.size() - 1] == ']')
         {
         algorithm = line.substr(1, line.size() - 2);
         alg_count = 0;
         continue;
         }

      std::vector<std::string> substr = parse(line);

#if DEBUG
      std::cout << "Testing: " << algorithm << std::endl;
#endif

      u32bit new_errors = 0;
      if(algorithm.find("Addition") != std::string::npos)
         new_errors = check_add(substr);

      if(algorithm.find("Subtraction") != std::string::npos)
         new_errors = check_sub(substr);

      if(algorithm.find("Multiplication") != std::string::npos)
         new_errors = check_mul(substr);

      if(algorithm.find("Division") != std::string::npos)
         new_errors = check_div(substr);

      if(algorithm.find("Modulo") != std::string::npos)
         new_errors = check_mod(substr);

      if(algorithm.find("LeftShift") != std::string::npos)
         new_errors = check_shl(substr);

      if(algorithm.find("RightShift") != std::string::npos)
         new_errors = check_shr(substr);

      alg_count++;
      errors += new_errors;

      if(new_errors)
         std::cout << "ERROR: BigInt " << algorithm << " failed test #"
                   << alg_count << std::endl;
      }

   return errors;
   }

u32bit check_add(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a + b;
   BigInt e = a;
   e += b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(d != e)
         std::cout << "ERROR: operator+/operator+= mismatch" << std::endl;
      return 1;
      }
   }

u32bit check_sub(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a - b;
   BigInt e = a;
   e -= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(d != e)
         std::cout << "ERROR: operator-/operator-= mismatch" << std::endl;
      return 1;
      }
   }

u32bit check_mul(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a * b;
   BigInt e = a;
   e *= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(d != e)
         std::cout << "ERROR: operator*/operator*= mismatch" << std::endl;
      return 1;
      }

   }

u32bit check_div(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a / b;
   BigInt e = a;
   e /= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator/ / operator/= mismatch" << std::endl;
      return 1;
      }

   }

u32bit check_mod(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   BigInt b(args[1]);
   BigInt c(args[2]);

   BigInt d = a % b;
   BigInt e = a;
   e %= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator%/operator%= mismatch" << std::endl;
      return 1;
      }
   }

u32bit check_shl(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   u32bit b = std::atoi(args[1].c_str());
   BigInt c(args[2]);

   BigInt d = a << b;
   BigInt e = a;
   e <<= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator<</operator<<= mismatch" << std::endl;
      return 1;
      }

   }

u32bit check_shr(const std::vector<std::string>& args)
   {
   BigInt a(args[0]);
   u32bit b = atoi(args[1].c_str());
   BigInt c(args[2]);

   BigInt d = a >> b;
   BigInt e = a;
   e >>= b;

   if(c == d && d == e)
      return 0;
   else
      {
      if(c != e)
         std::cout << "ERROR: operator>>/operator>>= mismatch" << std::endl;
      std::cout << c << std::endl;
      std::cout << d << std::endl;
      std::cout << e << std::endl;
      return 1;
      }
   }
